/**
 * @since 1.0.0
 */
import { AnalyzeDocumentCommand, AnalyzeExpenseCommand, AnalyzeIDCommand, CreateAdapterCommand, CreateAdapterVersionCommand, DeleteAdapterCommand, DeleteAdapterVersionCommand, DetectDocumentTextCommand, GetAdapterCommand, GetAdapterVersionCommand, GetDocumentAnalysisCommand, GetDocumentTextDetectionCommand, GetExpenseAnalysisCommand, GetLendingAnalysisCommand, GetLendingAnalysisSummaryCommand, ListAdaptersCommand, ListAdapterVersionsCommand, ListTagsForResourceCommand, StartDocumentAnalysisCommand, StartDocumentTextDetectionCommand, StartExpenseAnalysisCommand, StartLendingAnalysisCommand, TagResourceCommand, UntagResourceCommand, UpdateAdapterCommand, } from "@aws-sdk/client-textract";
import { Service } from "@effect-aws/commons";
import { Effect, Layer } from "effect";
import { AllServiceErrors } from "./Errors.js";
import * as Instance from "./TextractClientInstance.js";
import * as TextractServiceConfig from "./TextractServiceConfig.js";
const commands = {
    AnalyzeDocumentCommand,
    AnalyzeExpenseCommand,
    AnalyzeIDCommand,
    CreateAdapterCommand,
    CreateAdapterVersionCommand,
    DeleteAdapterCommand,
    DeleteAdapterVersionCommand,
    DetectDocumentTextCommand,
    GetAdapterCommand,
    GetAdapterVersionCommand,
    GetDocumentAnalysisCommand,
    GetDocumentTextDetectionCommand,
    GetExpenseAnalysisCommand,
    GetLendingAnalysisCommand,
    GetLendingAnalysisSummaryCommand,
    ListAdapterVersionsCommand,
    ListAdaptersCommand,
    ListTagsForResourceCommand,
    StartDocumentAnalysisCommand,
    StartDocumentTextDetectionCommand,
    StartExpenseAnalysisCommand,
    StartLendingAnalysisCommand,
    TagResourceCommand,
    UntagResourceCommand,
    UpdateAdapterCommand,
};
/**
 * @since 1.0.0
 * @category constructors
 */
export const makeTextractService = Effect.gen(function* () {
    const client = yield* Instance.TextractClientInstance;
    return Service.fromClientAndCommands(client, commands, {
        errorTags: AllServiceErrors,
        resolveClientConfig: TextractServiceConfig.toTextractClientConfig,
    });
});
/**
 * @since 1.0.0
 * @category models
 */
export class TextractService extends Effect.Tag("@effect-aws/client-textract/TextractService")() {
    static defaultLayer = Layer.effect(this, makeTextractService).pipe(Layer.provide(Instance.layer));
    static layer = (config) => Layer.effect(this, makeTextractService).pipe(Layer.provide(Instance.layer), Layer.provide(TextractServiceConfig.setTextractServiceConfig(config)));
    static baseLayer = (evaluate) => Layer.effect(this, makeTextractService).pipe(Layer.provide(Layer.effect(Instance.TextractClientInstance, Effect.map(TextractServiceConfig.toTextractClientConfig, evaluate))));
}
//# sourceMappingURL=TextractService.js.map