import { ServiceLogger } from "@effect-aws/commons";
import { Effect, FiberRef, Layer } from "effect";
import { dual } from "effect/Function";
import { globalValue } from "effect/GlobalValue";
/**
 * @since 1.0.0
 * @category textract service config
 */
const currentTextractServiceConfig = globalValue("@effect-aws/client-textract/currentTextractServiceConfig", () => FiberRef.unsafeMake({}));
/**
 * @since 1.0.0
 * @category textract service config
 */
export const withTextractServiceConfig = dual(2, (effect, config) => Effect.locally(effect, currentTextractServiceConfig, config));
/**
 * @since 1.0.0
 * @category textract service config
 */
export const setTextractServiceConfig = (config) => Layer.locallyScoped(currentTextractServiceConfig, config);
/**
 * @since 1.0.0
 * @category adapters
 */
export const toTextractClientConfig = Effect.gen(function* () {
    const { logger: serviceLogger, ...config } = yield* FiberRef.get(currentTextractServiceConfig);
    const logger = serviceLogger === true
        ? yield* ServiceLogger.toClientLogger(ServiceLogger.defaultServiceLogger)
        : (serviceLogger ? yield* ServiceLogger.toClientLogger(ServiceLogger.make(serviceLogger)) : undefined);
    return { logger, ...config };
});
//# sourceMappingURL=TextractServiceConfig.js.map